<?php

require_once('object_class.php');
require_once('op5/config.php');
require_once('op5/log.php');
require_once('op5/auth/Authorization.php');

/**
 * Users can log onto the Monitor GUI and see the objects that
 * they are configured to see. This differs from the concept of
 * contacts, who doesn't necessarily have access to the GUI.
 */
class user_class extends object_class {
	public $lazy_write = false; /**< If true, don't automatically generate auth_users.yml - performance option when handling many users */
	public $properties = array(
		'var_index' => array(
			'username' 		=> 'string',
			'realname' 		=> 'string',
			'password' 		=> 'password',
			'groups'   		=> 'mselect',
			'Access Levels' => 'norender' // deprecated, only for API compatibility purpouses
		),
		'default' => array(
			'realname'		=> '',
			'password' 		=> '',
			'groups'		=> '',
		),
		'special' => array(
			'groups'   		=> 'group'
		),
		'required' => array(),
		'label' => array(
			'realname' 		=> 'Real name',
			'password' 		=> 'Password<br>Repeat',
			'groups'   		=> 'Group Membership'
			),
		'topic' => 'username',
		'deprecated_names' => array(
			'Username' => 'username',
			'Realname' => 'realname',
			'Password' => 'password',
			'Role' => 'groups'
		)
	);

	public function get_properties() {
		$this->properties['special']['groups'] = array_combine(array_keys(self::get_groups()), array_keys(self::get_groups()));
		return $this->properties;
	}

	/** type name */
	public $type = 'user';
	/** doesn't actually exist, but it's the correct join pre-/suffix */
	public $db_table = 'username';

	private static $users = false;
	private static $groups = false;

	private static function get_groups() {
		if( is_array( self::$groups ) ) {
			return self::$groups;
		}
		return self::$groups = Op5Config::instance()->getConfig( 'auth_groups' );
	}

	private static function read_users()
	{
		self::get_groups();

		if( is_array( self::$users ) ) {
			return;
		}

		$auth_users = Op5Config::instance()->getConfig( 'auth_users' );

		self::$users = array();
		foreach( $auth_users as $username => $userdata ) {
			self::$users[ $username ] = $userdata;
		}
	}

	/**
	 * Read the rights and password files and statically store the data therein
	 */
	public static function get_users() {
		self::read_users();

		$user_objs = array();
		foreach(self::$users as $user => $userdata) {
			$obj = new self($user);
			$user_objs[] = $obj;
		}

		return $user_objs;
	}

	/**
	 * @param $username
	 * @return user_class|false
	 */
	public function get_object_by_name($username) {
		self::read_users();

		$this->id = $username;
		if (!isset(self::$users[$username]))
			return false;

		$this->obj = $this->get_default_object();
		if ($username !== false && $username !== 'new') {
			foreach( self::$users[$username] as $key => $value ) {
				$this->obj[$key] = $value;
			}
			$this->obj['username'] = $this->id;
			$this->properties['var_index']['username'] = 'hidden';
		} else {
			$this->obj['username'] = '';
		}

		/* Never push password or password hash to gui, it won't be updated if not explicitly updated */
		$this->obj['password'] = '';
		$this->obj['password_algo'] = '';

		return $this->obj;
	}

	public function __construct($id = false) {
		self::read_users();

		if (isset(self::$users[$id])) {
			$this->get_object_by_name($id);
		}
	}

	public function get_object_name() {
		return (isset($this->obj['username']) && $this->obj['username']) ? $this->obj['username'] : 'New User';
	}

	/**
	 * Write user files to disk
	 *
	 * If you set $lazy_write to true, this must be manually called, otherwise
	 * it will be called automatically at the end of relevant methods
	 *
	 * Depends on self::read_users(), which should be solved in
	 * self::__construct()
	 */
	public static function write_users() {
		Op5Config::instance()->setConfig('auth_users', self::$users );
	}

	/**
	 * Set user password
	 *
	 * @param $password
	 */
	public function set_password($password) {
		/* TODO: Which hashing algorithm? crypt is the only simple one available which has salt... */
		$this->obj['password'] = crypt( $password );
		$this->obj['password_algo'] = 'crypt';
		return true;
	}

	/**
	 * @param $user
	 * @return boolean
	 */
	public function save_object($user=true) {
		$log = Op5Log::instance('auth');
		self::get_groups(); /* For validation */

		if(!$this->validate_object()) {
			return false;
		}

		if (!empty($this->obj['password'])) {
			if (($chpass = $this->set_password($this->obj['password'])) !== true) {
				bug("Updating of password for user {$this->obj['username']} failed with return code $chpass");
				return false;
			}
		} else {
			/* If no password update, keep old... */
			$this->obj['password'] = self::$users[$this->obj['username']]['password'];
			$this->obj['password_algo'] = self::$users[$this->obj['username']]['password_algo'];
		}
		if (isset($this->obj['Access Levels'])) {
			$user_group = new usergroup_class();
			$user_group->obj['name'] = 'user_'.$this->obj['username'];
			$user_group->obj['access_rights'] = Op5Authorization::nagios_rights_to_op5auth($this->obj['Access Levels']);
			if (!$user_group->save_object()) {
				$this->validation_error('Access Levels', "Couldn't create group from levels");
				$this->validation_error_list = array_merge($this->validation_error_list, $user_group->validation_error_list);
				return false;
			}
			if (!isset($this->obj['groups']))
				$this->obj['groups'] = array();
			array_unshift($this->obj['groups'], $user_group->id);
			unset($this->obj['Access Levels']);
		}

		if (empty($this->validation_error_list)) {
			if( empty( $this->obj['username'] ) ) {
				return false;
			}

			self::$users[ $this->obj['username'] ] = $this->obj;

			if (!$this->lazy_write)
				self::write_users();
			return true;
		}
		return false;
	}

	function validate_object() {
		foreach ($this->obj as $k => $v) {
			$v_type = $this->properties['var_index'][$k];
			try {
				$class = $v_type . '_widget';
				require_once('widgets/'.$v_type.'.php');
				$class = new $class();
				$this->obj[$k] = $class->validate($this, $k, $this->obj[$k]);
			}
			catch (widget_validation_exception $ex) {
				if($this->id && $this->id != 'new' && 'password' == $k && empty($v[1]) && empty($v[2])) {
					// allow empty passwords when on acting on an already saved object
					$this->obj['password'] = null;
					continue;
				}

				return $this->validation_error($k, $ex->getMessage());
			}
		}
		if ($this->id == 'new' || !$this->id) {
			if (empty($this->obj['username'])) { // new user without username is no new user
				return true;
			}

			if (isset(self::$users[$this->obj['username']])) {
				return $this->validation_error('username', "A user named {$this->obj['username']} already exists. Not re-added.");
			}

			if (empty($this->obj['password'])) {
				return $this->validation_error('password', 'Password for new user empty. Not adding.');
			}
		}

		return true;
	}

	public function delete_object()
	{
		$this->banish_object();
	}

	public function banish_object($user=true)
	{
		$username = $this->get_object_name();

		unset(self::$users[$username]);

		if (!$this->lazy_write) {
			self::write_users();
		}
		return true;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = array();
		if ($this->id !== 'new' && !empty($this->id))
			$cmd_links['delete'] = "<a title='Delete this object' class='delete' href='access?action=delete&amp;delete=$this->id'>";
		return $cmd_links;
	}

	public function get_objects_for_user($obj_type=false)
	{
		if ($obj_type !== false && $obj_type !== $this->type)
			parent::get_objects_for_user($obj_type);
		else
			self::read_users();

		$usernames = array();
		foreach(self::$users as $user => $level) {
			$usernames[] = $user;
		}
		return $usernames;
	}

	public function normalize($obj = false, $abort_on_error=true)
	{
		if ($obj === false)
			$obj = $this->obj;

		if (isset($obj['Role'])) {
			if (!isset($obj['groups']))
				$obj['groups'] = array();
			array_unshift($obj['groups'], $obj['Role']);
			unset($obj['Role']);
		}
		parent::normalize($obj, $abort_on_error);
	}

	public function denormalize($obj = false) {
		$obj = parent::denormalize($obj);

		# Warning! Terrible magic here!
		# Password is never a meaningful value to show deserialized,
		# because it's always encrypted and secret, so unset it.
		unset($obj['password']);

		if (isset($obj['Roles']))
			$obj['Roles'] = implode(',', $obj['Roles']);
		$obj['Access Levels'] = array();

		return $obj;
	}

	/* Hack to make group validation work... */
	static public function object_exists( $obj, $id ) {
		if( $obj == 'group' ) return true;
		return parent::object_exists( $obj, $id );
	}
}
